﻿using System;
using System.IO;
using System.Windows.Forms;
using System.Collections.Generic;

using MiniLib.Model;

namespace MiniLib.Presentation
{
    public abstract class ControllerBase
    {
        private readonly IViewContainer _viewContainer;
        private readonly DocumentBase.DocumentManagerBase _documentManager;

        protected ControllerBase(IViewContainer viewContainer, DocumentBase.DocumentManagerBase documentManager)
        {
            if (viewContainer == null)
                throw new ArgumentNullException("viewContainer");
            if (documentManager == null)
                throw new ArgumentNullException("documentManager");
            _viewContainer = viewContainer;
            _documentManager = documentManager;
            _viewContainer.Load += delegate(object sender, EventArgs e)
                { Start(); };
            _viewContainer.FormClosing += delegate(object sender, FormClosingEventArgs e)
                { e.Cancel = !CanStop(); };
            _viewContainer.FormClosed += delegate(object sender, FormClosedEventArgs e)
                { Stop(); };
        }

        public IViewContainer ViewContainer
        {
            get { return _viewContainer; }
        }

        public DocumentBase.DocumentManagerBase DocumentManager
        {
            get { return _documentManager; }
        }

        public DocumentBase Document
        {
            get { return DocumentManager.Document; }
        }

        protected virtual void Start()
        {
            AdjustUserInterface();
            Application.Idle += new EventHandler(Application_Idle);
        }

        protected virtual bool CanStop()
        {
            if (!DocumentManager.DocumentExists)
                return true;
            return CloseDocument();
        }

        protected virtual void Stop()
        {
            Application.Idle -= new EventHandler(Application_Idle);
        }

        public virtual string GetStandardCaption()
        {
            if (DocumentManager.DocumentExists)
            {
                return Document.Name + (Document.IsDirty ? "*" : "");
            }
            else
            {
                return String.Empty;
            }
        }

        public virtual bool CanDoCommand(string commandName)
        {
            switch (commandName)
            {
                case "File":
                    return true;
                case "File.New":
                case "File.Open":
                    return !DocumentManager.DocumentExists;
                case "File.Close":
                case "File.Save":
                case "File.SaveAs":
                    return DocumentManager.DocumentExists;
                case "File.Exit":
                    return true;
                default:
                    return false;
            }
        }

        public virtual void DoCommand(string commandName)
        {
            switch (commandName)
            {
                case "File.New":
                    NewDocument();
                    break;
                case "File.Open":
                    OpenDocument();
                    break;
                case "File.Close":
                    CloseDocument();
                    break;
                case "File.Save":
                    SaveDocument(false);
                    break;
                case "File.SaveAs":
                    SaveDocument(true);
                    break;
                case "File.Exit":
                    Application.Exit();
                    break;
                default:
                    break;
            }
        }

        //  ----- Gestione Documento -----

        public virtual void NewDocument()
        {
            DocumentManager.NewDocument();
        }

        public virtual bool OpenDocument()
        {
            if (!DocumentManager.OpeningDocument())
                return false;
            using (CursorReplacer cursorReplacer = new CursorReplacer())
            {
                DocumentManager.OpenDocument();
            }
            return true;
        }

        public virtual bool SaveDocument(bool askForName)
        {
            if (!DocumentManager.SavingDocument(askForName))
                return false;
            using (CursorReplacer cursorReplacer = new CursorReplacer())
            {
                DocumentManager.SaveDocument();
            }
            return true;
        }

        public virtual bool CloseDocument()
        {
            if (!CanCloseDocument())
                return false;
            DocumentManager.CloseDocument();
            return true;
        }

        protected virtual bool CanCloseDocument()
        {
            if (!DocumentManager.DocumentExists)
                throw new ApplicationException("!DocumentManager.DocumentExists");
            if (Document.IsDirty)
            {
                DialogResult dialogResult = MessageBox.Show(String.Format("Salvo le modifiche fatte a {0}?", Document.Name),
                    "Document Closing", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
                switch (dialogResult)
                {
                    case DialogResult.Yes:
                        SaveDocument(false);
                        break;
                    case DialogResult.No:
                        return DocumentManager.CanCloseNotSavedDocument();
                    case DialogResult.Cancel:
                        return false;
                }
            }
            return true;
        }

        //  --- Gestione User Interface ---

        private void Application_Idle(object sender, EventArgs e)
        {
            UpdateUserInterface();
        }

        protected virtual void AdjustUserInterface()
        {
            _viewContainer.AdjustUserInterface(this);
        }

        protected virtual void UpdateUserInterface()
        {
            _viewContainer.UpdateUserInterface(this);
        }

        public void AdjustMenuStrip(MenuStrip menuStrip)
        {
            if (menuStrip != null)
                AdjustItems(menuStrip.Items, String.Empty);
        }

        public void UpdateMenuStrip(MenuStrip menuStrip)
        {
            if (menuStrip != null)
                UpdateMenuItems(menuStrip.Items);
        }

        #region Gestione generalizzata menù

        private readonly Dictionary<ToolStripItem, string> _toolStripItemCommandDictionary =
            new Dictionary<ToolStripItem, string>();

        private void AdjustItems(ToolStripItemCollection toolStripItems, string preCommandName)
        {
            foreach (ToolStripItem toolStripItem in toolStripItems)
            {
                if (IsToConsider(toolStripItem))
                {
                    ToolStripMenuItem toolStripMenuItem = toolStripItem as ToolStripMenuItem;
                    if (!_toolStripItemCommandDictionary.ContainsKey(toolStripMenuItem))
                    {
                        toolStripMenuItem.Click += new EventHandler(ToolStripMenuItem_Click);
                        toolStripMenuItem.DropDownOpening += new EventHandler(ToolStripMenuItem_DropDownOpening);
                    }
                    string commandName = (toolStripMenuItem.Tag as string);
                    if (String.IsNullOrEmpty(commandName))
                    {
                        commandName = preCommandName + NormalizeMenuText(toolStripMenuItem.Text);
                    }
                    _toolStripItemCommandDictionary[toolStripMenuItem] = commandName;
                    AdjustItems(toolStripMenuItem.DropDownItems, commandName + ".");
                }
            }
        }

        private string GetCommand(ToolStripItem toolStripItem)
        {
            return _toolStripItemCommandDictionary[toolStripItem];
        }

        private string NormalizeMenuText(string menuText)
        {
            return menuText.Replace("&", "").Replace(".", "").Replace(" ", "");
        }

        private bool IsToConsider(ToolStripItem toolStripItem)
        {
            //  Non devo prendere in considerazione le voci del menù di sistema: MdiControlStrip.SystemMenuItem
            if (toolStripItem.GetType() != typeof(ToolStripMenuItem))
                return false;
            ToolStripMenuItem toolStripMenuItem = (ToolStripMenuItem) toolStripItem;
            //  Non devo prendere in considerazione le voci del menù Window inserite e gestite dal sistema
            if (toolStripMenuItem.IsMdiWindowListEntry)
                return false;
            if (toolStripMenuItem.Text == "&More Windows...")
                return false;
            return true;
        }

        private void ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem senderToolStripMenuItem = sender as ToolStripMenuItem;
            if (senderToolStripMenuItem == null) return;
            string commandName = GetCommand(senderToolStripMenuItem);
            DoCommand(commandName);
        }

        private void ToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            ToolStripMenuItem senderToolStripMenuItem = sender as ToolStripMenuItem;
            if (senderToolStripMenuItem == null) return;
            UpdateMenuItems(senderToolStripMenuItem.DropDownItems);
        }

        private void UpdateMenuItems(ToolStripItemCollection items)
        {
            foreach (ToolStripItem toolStripItem in items)
            {
                if (IsToConsider(toolStripItem))
                {
                    ToolStripMenuItem toolStripMenuItem = toolStripItem as ToolStripMenuItem;
                    string commandName = GetCommand(toolStripMenuItem);
                    toolStripMenuItem.Enabled = CanDoCommand(commandName);
                }
            }
        }

        #endregion
    }
}
