﻿using System;
using System.Drawing;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;
using System.Windows.Forms;
using System.ComponentModel;

using MiniLib.Model;

namespace MiniLib.Presentation
{
    public partial class SelectionControl : UserControl
    {
        private bool _adjustComboBoxesWidth = true;
        private readonly Selectors _selectors = new Selectors();

        public SelectionControl()
        {
            InitializeComponent();
        }

        public bool AdjustComboBoxesWidth
        {
            get { return _adjustComboBoxesWidth; }
            set { _adjustComboBoxesWidth = value; }
        }

        public object GetSelectedItem()
        {
            return GetSelectedItem(0);
        }

        public object GetSelectedItem(int index)
        {
            return _selectors[index].SelectedItem;
        }

        public object GetSelectedItem(string name)
        {
            return _selectors[name].SelectedItem;
        }

        public void SetSelection(IEnumerable items)
        {
            ClearSelections();
            AddSelection(items);
        }

        public void SetSelection(string name, string caption, IEnumerable items)
        {
            ClearSelections();
            AddSelection(name, caption, items);
        }

        public void AddSelection(IEnumerable items)
        {
            string name = String.Format("${0}", _selectors.Count);
            string caption = String.Format("Seleziona l'elemento {0}:", _selectors.Count);
            AddSelection(name, caption, items);
        }

        public void AddSelection(string name, string caption, IEnumerable items)
        {
            if (String.IsNullOrEmpty(name))
                throw new ArgumentNullException("String.IsNullOrEmpty(name)");
            if (String.IsNullOrEmpty(caption))
                throw new ArgumentNullException("String.IsNullOrEmpty(caption)");
            if (items == null)
                throw new ArgumentNullException("items");
            Selector selector = new Selector(name, caption, items);
            _selectors.Add(selector, AdjustComboBoxesWidth);
            _tableLayoutPanel.Controls.Add(selector.Label);
            _tableLayoutPanel.Controls.Add(selector.ComboBox);
            //  Ci deve essere sempre una riga in più
            _tableLayoutPanel.RowCount++;
        }

        public void ClearSelections()
        {
            _selectors.Clear();
            _tableLayoutPanel.Controls.Clear();
            _tableLayoutPanel.RowCount = 1;
        }

        private class Selectors
        {
            private readonly List<Selector> _selectorsList = new List<Selector>();
            private readonly Dictionary<string, Selector> _selectorsDictionary = new Dictionary<string, Selector>();

            public int Count
            {
                get { return _selectorsList.Count; }
            }

            public Selector this[int index]
            {
                get { return _selectorsList[index]; }
            }

            public Selector this[string name]
            {
                get { return _selectorsDictionary[name]; }
            }

            public void Clear()
            {
                _selectorsList.Clear();
                _selectorsDictionary.Clear();
            }

            public void Add(Selector selector, bool adjustComboBoxesWidth)
            {
                if (_selectorsDictionary.ContainsKey(selector.Name))
                    throw new ArgumentException("Nome di selezione ripetuto!");
                _selectorsList.Add(selector);
                _selectorsDictionary.Add(selector.Name, selector);
                if (adjustComboBoxesWidth && Count > 1)
                    AdjustComboBoxesWidth();
            }

            private void AdjustComboBoxesWidth()
            {
                int maxComboBoxWidth = 0;
                foreach (Selector selector in _selectorsList)
                {
                    maxComboBoxWidth = Math.Max(maxComboBoxWidth, selector.ComboBox.Width);
                }
                foreach (Selector selector in _selectorsList)
                {
                    selector.ComboBox.Width = maxComboBoxWidth;
                }
            }
        }

        private class Selector
        {
            private readonly string _name;
            private readonly Label _label;
            private readonly ComboBox _comboBox;

            public Selector(string name, string caption, IEnumerable items)
            {
                _name = name;

                _label = new Label();
                _label.Text = caption;
                _label.AutoSize = true;
                _label.Margin = new Padding(10);
                _label.Anchor = AnchorStyles.Right;
                _label.TextAlign = ContentAlignment.MiddleRight;

                _comboBox = new ComboBox();
                _comboBox.Margin = new Padding(10);
                _comboBox.Anchor = AnchorStyles.Left;
                _comboBox.DropDownStyle = ComboBoxStyle.DropDownList;
                LoadComboBoxItems(items);
                AdjustComboBoxWidth();
            }

            public string Name
            {
                get { return _name; }
            }

            public Label Label
            {
                get { return _label; }
            }

            public ComboBox ComboBox
            {
                get { return _comboBox; }
            }

            public object SelectedItem
            {
                get { return _comboBox.SelectedItem; }
            }

            private void LoadComboBoxItems(IEnumerable items)
            {
                _comboBox.Items.Clear();
                foreach (object item in items)
                {
                    _comboBox.Items.Add(item);
                }
                if (_comboBox.Items.Count > 0)
                    _comboBox.SelectedIndex = 0;
            }

            private void AdjustComboBoxWidth()
            {
                int extraWidth = SystemInformation.VerticalScrollBarWidth;
                int optimalTextWidth = _comboBox.Width - extraWidth;
                using (Graphics graphics = _comboBox.CreateGraphics())
                {
                    foreach (object item in _comboBox.Items)
                    {
                        SizeF size = graphics.MeasureString(_comboBox.GetItemText(item), _comboBox.Font);
                        optimalTextWidth = Math.Max(optimalTextWidth, (int) Math.Ceiling(size.Width));
                    }
                }
                _comboBox.Width = optimalTextWidth + extraWidth;
            }
        }
    }
}
